@@ -25,6 +25,7 @@
 #include "libglade-buddy.h"
 #include "cell-renderer-uri.h"
 #include "md5-utils.h"
+#include "distribution.h"
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -188,22 +189,24 @@ load_products_xml (BugzillaBTS *bts, xmlDoc *doc)
 
 			prod->name        = XML_NODE_GET_PROP (node, "name");
 			prod->description = XML_NODE_GET_PROP (node, "description");
+			prod->versions    = NULL;
 			prod->components  = g_hash_table_new (g_str_hash, g_str_equal);
 
 			bugzilla_bts_insert_product (bts, prod);
 
 			for (cur = node->children; cur; cur = cur->next) {
 				d(g_print ("\t\t%s\n", node->name));
-				if (strcmp (cur->name, "component"))
-					continue;
+				if (!strcmp (cur->name, "component")) {
+					comp = g_new0 (BugzillaComponent, 1);
+					comp->product = prod;
 				
-				comp = g_new0 (BugzillaComponent, 1);
-				comp->product = prod;
-				
-				comp->name        = XML_NODE_GET_PROP (cur, "value");
-				comp->description = XML_NODE_GET_PROP (cur, "description");
+					comp->name        = XML_NODE_GET_PROP (cur, "value");
+					comp->description = XML_NODE_GET_PROP (cur, "description");
 
-				bugzilla_product_insert_component (prod, comp);
+					bugzilla_product_insert_component (prod, comp);
+				} else if (!strcmp (cur->name, "version")) {
+					prod->versions = g_slist_append(prod->versions, XML_NODE_GET_PROP (cur, "value"));
+				}
 			}
 		}
 	}
@@ -551,7 +554,7 @@ static xmlDoc *
 load_bugzilla_xml_file (BugzillaXMLFile *xml_file)
 {
 	xmlDoc *doc = NULL;
-	struct stat statbuf;
+	struct stat statbuf, statbuf2;
 
 	if (!stat (xml_file->tmp_path, &statbuf) && statbuf.st_size) {
 		if (!rename (xml_file->tmp_path, xml_file->cache_path)) {
@@ -560,7 +563,11 @@ load_bugzilla_xml_file (BugzillaXMLFile *xml_file)
 	}
 
 
-	if (!doc)
+	if (!doc && !stat (xml_file->cache_path, &statbuf) && !stat(xml_file->system_path, &statbuf2) &&
+	    difftime (statbuf.st_mtime,statbuf2.st_mtime) > 0) 
+		doc = xmlParseFile (xml_file->cache_path);
+	
+	if (!doc) 
 		doc = xmlParseFile (xml_file->system_path);
 
 	return doc;
@@ -930,7 +937,7 @@ load_bugzillas (void)
 	d(g_list_foreach (druid_data.dldests, (GFunc)p_string, NULL));
 }
 
-#define ALL_APPLICATIONS_URI "all-applications:///"
+#define ALL_APPLICATIONS_URI "applications-all-users:///"
 #define APPLET_REQUIREMENTS \
 	"has_all (repo_ids, ['IDL:Bonobo/Control:1.0'," \
 	"		     'IDL:GNOME/Vertigo/PanelAppletShell:1.0']) && " \
@@ -940,11 +947,13 @@ load_bugzillas (void)
 #define BUGZILLA_PRODUCT                "X-GNOME-Bugzilla-Product"
 #define BUGZILLA_COMPONENT              "X-GNOME-Bugzilla-Component"
 #define BUGZILLA_EMAIL                  "X-GNOME-Bugzilla-Email"
+#define BUGZILLA_VERSION                "X-GNOME-Bugzilla-Version"
 #define BUGZILLA_OTHER_BINARIES         "X-Gnome-Bugzilla-OtherBinaries"
 
 #define BA_BUGZILLA_BUGZILLA            "bugzilla:bugzilla"
 #define BA_BUGZILLA_PRODUCT             "bugzilla:product"
 #define BA_BUGZILLA_COMPONENT           "bugzilla:component"
+#define BA_BUGZILLA_VERSION             "bugzilla:version"
 #define BA_BUGZILLA_EMAIL               "bugzilla:email"
 #define BA_BUGZILLA_OTHER_BINARIES      "bugzilla:other_binaries"
 
@@ -954,6 +963,7 @@ bugzilla_application_new (const char *name,
 			  const char *bugzilla, 
 			  const char *product,
 			  const char *component, 
+			  const char *version, 
 			  const char *email, 
 			  const char *icon,
 			  const char *program,
@@ -973,6 +983,7 @@ bugzilla_application_new (const char *name,
 	app->bugzilla    = g_strdup (bugzilla);
 	app->product     = g_strdup (product);
 	app->component   = g_strdup (component);
+	app->version     = g_strdup (version);
 	app->email       = g_strdup (email);
 
 	if (icon && *icon != G_DIR_SEPARATOR)
@@ -995,7 +1006,6 @@ bugzilla_application_new (const char *name,
 		g_error_free (error);
 	}
 	
-	druid_data.applications = g_slist_prepend (druid_data.applications, app);
 
 	if (program) {
 		g_shell_parse_argv (program, &i, &programv, NULL);
@@ -1006,11 +1016,21 @@ bugzilla_application_new (const char *name,
 			if (g_hash_table_lookup (druid_data.program_to_application, s) == NULL) {
 				d(g_print ("adding app: %s\n", s));
 				g_hash_table_insert (druid_data.program_to_application, g_strdup (s), app);
+				druid_data.applications = g_slist_prepend (druid_data.applications, app);
+			} else {
+				g_free (app);
+				return;
 			}
+		} else {
+			g_free (app);
+			return;
 		}
 		if (programv)
 			g_strfreev (programv);
+	} else {
+		druid_data.applications = g_slist_prepend (druid_data.applications, app);
 	}
+		
 
 	if (other_programs) {
 		programv = g_strsplit (other_programs, ";", -1);
@@ -1072,6 +1092,7 @@ load_applets (GnomeIconTheme *git)
 			bonobo_server_info_prop_lookup (info, BA_BUGZILLA_BUGZILLA,  NULL),
 			bonobo_server_info_prop_lookup (info, BA_BUGZILLA_PRODUCT,   NULL),
 			bonobo_server_info_prop_lookup (info, BA_BUGZILLA_COMPONENT, NULL),
+			bonobo_server_info_prop_lookup (info, BA_BUGZILLA_VERSION, NULL),
 			bonobo_server_info_prop_lookup (info, BA_BUGZILLA_EMAIL,     NULL),
 			bonobo_server_info_prop_lookup (info, "panel:icon", NULL),
 			NULL,
@@ -1125,6 +1146,7 @@ visit_cb (const char *rel_path,
 		gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA),
 		gnome_desktop_item_get_string (ditem, BUGZILLA_PRODUCT),
 		gnome_desktop_item_get_string (ditem, BUGZILLA_COMPONENT),
+		gnome_desktop_item_get_string (ditem, BUGZILLA_VERSION),
 		gnome_desktop_item_get_string (ditem, BUGZILLA_EMAIL),
 		icon,
 		gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC),
@@ -1182,6 +1204,13 @@ update_severity (GtkWidget *w, gpointer data)
 	druid_data.severity = (char *)data;
 }
 
+static void
+update_version (GtkWidget *w, gpointer data)
+{
+	druid_data.version = (char *)data;
+}
+
+
 static void
 add_severity (char *s, GtkMenu *m)
 {
@@ -1217,7 +1246,9 @@ add_product (gpointer key, BugzillaProduct *p, GtkListStore *store)
 static void
 add_products (gpointer key, BugzillaBTS *bts, GtkListStore *store)
 {
-	g_hash_table_foreach (bts->products, (GHFunc)add_product, store);
+	if (bts->products != NULL) {
+		g_hash_table_foreach (bts->products, (GHFunc)add_product, store);
+	}
 }
 
 static void
@@ -1232,6 +1263,32 @@ add_application (BugzillaApplication *app, GtkListStore *store)
 			    -1);
 }
 
+void
+update_version_menu (BugzillaProduct *prod)
+{
+	GSList *versions;
+	GtkWidget *w;
+	GtkWidget *m, *c;
+
+	versions = prod->versions;
+
+	m = gtk_menu_new ();
+	c = GET_WIDGET ("the-version-menu");
+        gtk_option_menu_set_menu (GTK_OPTION_MENU (c), m);
+
+	while (versions != NULL) {
+		w = gtk_menu_item_new_with_label ((gchar*)versions->data);
+		g_signal_connect (G_OBJECT (w), "activate",
+				  G_CALLBACK (update_version),
+				  versions->data);
+		gtk_widget_show (w);
+		gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
+		versions = versions->next;
+	}
+}
+
+
+
 static gboolean
 unselect (gpointer data)
 {
@@ -1406,7 +1463,7 @@ generate_email_text (gboolean include_headers)
 	email = g_string_new (NULL);
 
 	subject    = buddy_get_text ("desc-subject");
-	version    = buddy_get_text ("the-version-entry");
+	version    = druid_data.version ? druid_data.version : "unspecified";
 
 	s          = buddy_get_text ("desc-text");
 	body = format_for_width (s);
@@ -1417,12 +1474,17 @@ generate_email_text (gboolean include_headers)
 	g_free (s);
 
 	severity = druid_data.severity ? druid_data.severity : "Normal";
-
+	
 	if (include_headers)
 		g_string_append_printf (email, "Subject: %s\n\n", subject);
 
+	get_distro_name();
+	if (druid_data.distro)
+		g_string_append_printf (email, "Distribution: %s\n", druid_data.distro);
 	if (!is_bugzilla) {
 		g_string_append (email, "(This bug report was generated by Bug Buddy " VERSION ")\n");
+		g_string_append (email, "Description:\n");
+		
 	} else if (druid_data.product->bts->submit_type == BUGZILLA_SUBMIT_DEBIAN) {
 		/* This is what gnome.org and ximian.com use */
 		g_string_append_printf (email, "Package: %s\n", druid_data.product->name);
@@ -1432,11 +1494,17 @@ generate_email_text (gboolean include_headers)
 					? druid_data.gnome_platform_version
 					: "",
 					version);
-		if (druid_data.gnome_vendor)
-			g_string_append_printf (email, "os_details: %s\n", druid_data.gnome_vendor);
+		if (druid_data.gnome_distributor)
+			g_string_append_printf (email, "Gnome-Distributor: %s\n", druid_data.gnome_distributor);
 		g_string_append_printf (email, "Synopsis: %s\n", subject);
 		g_string_append_printf (email, "Bugzilla-Product: %s\n", druid_data.product->name);
 		g_string_append_printf (email, "Bugzilla-Component: %s\n", druid_data.component->name);
+		g_string_append_printf (email, "Bugzilla-Version: %s\n", version);
+
+		if (druid_data.bug_type == BUG_DOC)
+			g_string_append_printf (email, "Bugzilla-Keyword: doc\n");
+		if (druid_data.bug_type == BUG_I18N)
+			g_string_append_printf (email, "Bugzilla-Keyword: i18n\n");
 		if (druid_data.gnome_version)
 			g_string_append_printf (email, "BugBuddy-GnomeVersion: %s\n", druid_data.gnome_version);
 		g_string_append (email, "Description:\n");
